@@ -44,6 +44,7 @@
 (require 'org-srs-item)
 (require 'org-srs-review)
 (require 'org-srs-review-rate)
+(require 'org-srs-review-strategy)
 (require 'org-srs-property)
 (require 'org-srs-query)
 
@@ -287,6 +288,16 @@ from a large set of review items."
              (setf (org-srs-review-cache-nth ,n ,list ',null) ,default)
            ,value)))))
 
+(define-advice org-srs-query-item-p (:around (fun . (predicate &rest item)) org-srs-review-cache)
+  (if (org-srs-review-cache-active-p)
+      (cl-loop for (cached-predicate . table) in (org-srs-review-cache-queries (org-srs-review-cache))
+               if (equal predicate cached-predicate)
+               return (gethash item table)
+               else if (and (consp cached-predicate) (eq (car cached-predicate) 'and) (cl-find predicate (cdr cached-predicate) :test #'equal))
+               when (gethash item table) return t
+               finally (cl-return (apply fun predicate item)))
+    (apply fun predicate item)))
+
 (define-advice org-srs-item-due-times (:around (fun n &rest args) org-srs-review-cache)
   (if (and (org-srs-review-cache-active-p) args)
       (let* ((args (apply #'org-srs-review-cache-item args))
@@ -315,21 +326,23 @@ from a large set of review items."
   (cl-destructuring-bind (predicate &optional (source (or (buffer-file-name) default-directory))) args
     (if (org-srs-review-cache-active-p)
         (let ((result (org-srs-review-cache-query predicate source)))
-          (if (eq result org-srs-review-cache-null)
-              (let* ((cache (or (org-srs-review-cache) (setf (org-srs-review-cache) (make-org-srs-review-cache :source source))))
-                     (query (cl-multiple-value-bind (due-predicate rest-predicate) (org-srs-review-cache-query-predicate-due predicate)
-                              (if due-predicate
-                                  (let ((tree (org-srs-review-cache-pending-queue due-predicate)))
-                                    (cl-flet* ((cache-marker (&optional (item (org-srs-review-cache-item)))
-                                                 (setf (gethash item (org-srs-review-cache-markers cache)) (point-marker)))
-                                               (cache-due-time (&aux (item (org-srs-review-cache-item)))
-                                                 (avl-tree-enter tree (cons (org-srs-item-due-time) item))
-                                                 (cache-marker item)
-                                                 nil))
-                                      `(and ,rest-predicate (or (and ,due-predicate ,#'cache-marker) ,#'cache-due-time))))
-                                rest-predicate))))
-                (setf (org-srs-review-cache-query predicate source) (funcall fun query source)))
-            result))
+          (when (eq result org-srs-review-cache-null)
+            (let* ((cache (or (org-srs-review-cache) (setf (org-srs-review-cache) (make-org-srs-review-cache :source source))))
+                   (query (cl-multiple-value-bind (due-predicate rest-predicate) (org-srs-review-cache-query-predicate-due predicate)
+                            (if due-predicate
+                                (let ((tree (org-srs-review-cache-pending-queue due-predicate)))
+                                  (cl-flet* ((cache-marker (&optional (item (org-srs-review-cache-item)))
+                                               (setf (gethash item (org-srs-review-cache-markers cache)) (point-marker)))
+                                             (cache-due-time (&aux (item (org-srs-review-cache-item)))
+                                               (avl-tree-enter tree (cons (org-srs-item-due-time) item))
+                                               (cache-marker item)
+                                               nil))
+                                    `(and ,rest-predicate (or (and ,due-predicate ,#'cache-marker) ,#'cache-due-time))))
+                              rest-predicate))))
+              (setf (org-srs-review-cache-query predicate source) (funcall fun query source)
+                    result (org-srs-review-cache-query predicate source)))
+            (cl-assert (not (eq result org-srs-review-cache-null))))
+          result)
       (funcall fun predicate source))))
 
 (define-advice org-srs-review-start (:around (fun &rest args) org-srs-review-cache)
@@ -429,12 +442,19 @@ from a large set of review items."
 (define-advice org-srs-query (:around (fun &rest args) org-srs-query-predicate@org-srs-review-cache)
   (org-srs-review-cache-without-query-predicate (apply fun args)))
 
-(define-advice org-srs-query-item-p (:around (fun . (predicate &rest item)) org-srs-query-predicate@org-srs-review-cache)
-  (if (org-srs-review-cache-active-p)
-      (progn
-        (cl-assert (not (bound-and-true-p org-srs-query-predicate@org-srs-review-cache)))
-        (if item (apply predicate item) (apply fun predicate item)))
-    (apply fun predicate item)))
+(defvar org-srs-review-cache-strategy-items nil
+  "Dynamically bound hash table to cache items for review strategies.")
+
+(cl-defmethod org-srs-review-strategy-items :around (_state _strategy &context (org-srs-review-cache-strategy-items null) &rest _args)
+  "Bind `org-srs-review-cache-strategy-items' to a new hash table."
+  (let ((org-srs-review-cache-strategy-items (make-hash-table :test #'equal)))
+    (cl-call-next-method)))
+
+(cl-defmethod org-srs-review-strategy-items :around (state strategy &context (org-srs-review-cache-strategy-items hash-table) &rest args)
+  "Try to return cached review items in STATE matching STRATEGY with ARGS."
+  (org-srs-review-cache-ensure-gethash
+   (cl-list* org-srs-review-strategy-due-predicate state strategy args)
+   org-srs-review-cache-strategy-items (cl-call-next-method)))
 
 (provide 'org-srs-review-cache)
 ;;; org-srs-review-cache.el ends here
